//
// main.cpp
//
// file structure (in order of edits/change frequency):
//
//    effect.cpp: all code specific to "this" graphic
//                can be defined out
//                changes app to app
//
//    main.cpp:   basic minimal framework stuff
//                skybox/star/particle environment (key 'u' Universe)
//                generic light (toggle / key 's' Spotlight)
//
//    std.cpp (include file) has rarely-changing framework functions, callbacks, etc.
//
//    hardware.cpp (hw) refers to a singleton global containing:
//                 screen class w/ multiple viewports w/ independent controllable cameras 
//                 mouse, key, time, userPrefs and the FMOD sound system
//
//    the rest are misc class files (text/texture/skybox/splinter/particles/mesh...)
//
//------------------------------------------------------------------------------

#define haseffect

#include "main.h"
#include "billboard.h"
#include "config.h"
#include "dialog.h"
#include "fog.h"
#include "light.h"
#include "listbox.h"
#include "random.h"
#include "skybox.h"
#include "splinter.h"
#include "starfield.h"
#include "text.h"
#include "texture.h"

#if defined haseffect
#include "effect.h"
#endif

//------------------------ global framework objects

Hw *hw;            // hardware pointer
Billboard sunBB;   // sun/moon/spotlight @ light0
Config config;     // load file and update prefs
Dialog dialogObject, *dialog=&dialogObject; // old popup menu system
Fog fog;
Light light;       // gl light0
Listbox listbox;   // info.txt file scroller
Randoms randoms;   // manager for random splinter, lightmap, blank... effects
Scroller scroller; // one-line text/hint scroller
Skybox skybox;
Splinter splinter;     // solid mesh -> independent moving quads
Starfield  starfield;  // far flat star glpoints
Starfield2 starfield2; // near particle textured quads
Text ann, clock, note, title, splash, dialogText; // feedback annotation, fps and other config'd text objects
Texture fontT, controlsT, sphereT, sunT; // starT, smokeT, 

#if defined haseffect
Effect effect; // main "this app" effect class
#endif

char s[80]={"PhysicZ"}; // generic string

float lastscrolltime=0;

//--------------------------------------------------------------------

void main(int argc, char** argv) {
	hw=Hw::get(); // singleton global w/ all hardware and prefs
	hw->ann=&ann; // reference standard annotation

	strcpy(hw->pref.title,s); // set app name

	strcpy(s,"Starting up ");         // make log entry
	strcat(s,hw->pref.title);
	strcat(s,"...");
	log_message(s);

	hw->pref.set(&ann);          // load prefs from file or hardcoded
	config.load(hw->pref.title); // overide w/ config

	glut_init(argc, argv);
}

//--------------------------------------------------------------------

#include "std.cpp"  // all standard rarely-changing stuff

//--------------------------------------------------------------------
//--------------------------------------------------------------------
//--------------------------------------------------------------------

void annot_this(void){ // 2d ortho draw in current viewport over 3d stuff

#if defined haseffect
	effect.annot(&ann);
#endif
}

//--------------------------------------------------------------------

void close_this(void) { // 1 time final shutdown
	log_message("Close this...");
	starfield.destroy();

#if defined haseffect
	effect.close();

	dialog->clear();

#endif
}

//--------------------------------------------------------------------
	
// old dialog system
#include "dialog_main.cpp"

//--------------------------------------------------------------------

void draw_this(void){ // main 3d perspective draw in current viewport
	if (hw->pref.lights) {
		light.set_xyz(-hw->pref.worldSize*1.5,
			            hw->pref.worldSize*.9, 0);
		//light.set_xyz(sinf(hw->time.time*.025)*hw->pref.worldSize*1.5,
		//	                                  hw->pref.worldSize*1,
		//				  cosf(hw->time.time*.025)*hw->pref.worldSize*1.5);
		light.amb=hw->pref.ambient;
		light.diff=hw->pref.diffuse;
		light.refl=hw->pref.reflective;
		light.spec=hw->pref.specular;
		light.exponent=hw->pref.exponent;
		light.cutoff=hw->pref.cutoff;
		glMateriali(GL_FRONT_AND_BACK,GL_SHININESS,hw->pref.shininess); // 5 bigspot-50 smallspot
		light.setup();
	}
	if (hw->pref.universe<4) { // render?
		sunBB.vis.set(); // set renderstate for sun
		glPushMatrix();
			glTranslatef(light.x,light.y,light.z);
			sunBB.draw(1);
			glPopMatrix();
		sunBB.vis.unset();// unset render state
	}

#if defined haseffect
	effect.draw(&ann);
#endif
}

//--------------------------------------------------------------------

void init_this(void) { // 1 time init
	float meter;

	log_message("Init this...");

	hw->pref.worldSize=1000; // world spec -----------------------------
	meter=hw->pref.meter=1;
	hw->screen.nearz=.1;//.002;
	hw->screen.farz=hw->pref.worldSize*3;//3.57;
	hw->screen.set_nviewports(hw->pref.viewportset); // set 1 viewport

	//light.set_params(.1,.9,.9,.99);// main light ----------------
	light.set_params(hw->pref.ambient,hw->pref.diffuse,hw->pref.specular,hw->pref.reflective);
	light.exponent  =hw->pref.exponent;
	light.cutoff    =hw->pref.cutoff;

	sunBB.vis.reset(); // sun billboard -------------------------
	sunBB.sizex=.04*hw->pref.worldSize;
	sunBB.sizey=sunBB.sizex;
	sunBB.vis.alphatest=1; 
	sunBB.vis.alphafunc=.01; 
	sunBB.vis.fog=0;
	sunBB.vis.image=1;
	sunBB.vis.lights=0;
	sunBB.vis.tex=&sunT;
	if (sunT.load_tex("images/sun_128.bmp", GL_RGBA, -1, 0, 0))
		log_message("Texture sun load problem **********");

	if (hw->pref.skipSplash) scroller.msg("PhysicZ - GameDev's 'Physics' Contest entry - Copyright (C) 2002 by George Ziniewicz - zintel.com");

	skybox.gap=1.5; // 50% > WorldSize
	skybox.init();  // load textures

	log_message("Init starfields..."); // static displaylist far starfield  ------
	starfield.n=hw->pref.nstars;
	starfield.nlevel=2;
	starfield.size=hw->pref.worldSize*2;
	starfield.hollow=.5*starfield.size;
	starfield.blend=1; starfield.depthtest=0; starfield.depthwrite=0;
	starfield.type=1; // 0-round/1-cube/2-dome
	starfield.init();// init volume univ

	starfield2.n=500; // brownian floating infinite particles ----------------
	starfield2.size=hw->pref.worldSize*.25;
	starfield2.hollow=0*starfield2.size;
	starfield2.blend=1; starfield2.blendFunc0=2; starfield2.blendFunc1=3;
	starfield2.depthtest=1; starfield2.depthwrite=0;
	starfield2.type=1; // 0-round/1-cube/2-dome
	starfield2.tex=&sphereT;
	starfield2.sizei=.08*meter;
	starfield2.init();// init volume univ

	log_message("Init std textures...");

	//Text::charInX=16;	Text::charInY=16; Text::txoffy=0;// setup font
	//if (fontT.load_tex("images/lucida-futura-smooth256.bmp", GL_LUMINANCE_ALPHA, -1, 0, 0)) { // textures -----

	Text::charInX=13;	Text::charInY=8; // setup font
	if (fontT.load_tex("images/lucidaconsole.bmp", GL_LUMINANCE_ALPHA, -1, 0, 0)) { // textures -----
		log_message("Texture font load problem **********");
		glut_shutdown();
	}
	if (sphereT.make_tex(SPHERE,128,128, GL_LUMINANCE_ALPHA,1)) log_message("Cant make sphere texture **********");

	if (controlsT.load_tex("images/controls_128.bmp", GL_RGBA, -1, 0, 0))
		log_message("Texture controls load problem **********");

	//if (smokeT.make_tex(SMOKE,  128,128,GL_LUMINANCE_ALPHA,1)) log_message("Cant make smoke texture **********");
	//if (starT.make_tex(STAR,    128,128,GL_LUMINANCE_ALPHA,1)) log_message("Cant make star texture **********");

	Widget::tex=&controlsT; // ----------- dialog
	Widget::text=&dialogText;
	//dialogText.x=0;
	//dialogText.y=0;
	dialogText.alphafunc=.05;
	dialogText.blend=0;
	dialog->hasColor=0;
	dialog->hasLighting=0;
	dialog->text=&dialogText;
	dialog_init(MAIN,1);
	hw->pref.dialogs=0;

	log_message("Init Splinter...");
	splinter.c2=.2;  // 20% fade effect
	splinter.plus=0; // only draw splinter (vs. plus current frame)
	if (splinter.init()) log_message("Cant make splinter texture **********"); // make texture

#if defined haseffect
	effect.sphereT=&sphereT;
	effect.init(&ann);
#endif
}

//--------------------------------------------------------------------
	
void key_this(void){
	static int reverbMode=0;

	if ((hw->pref.skipSplash==0)||(hw->pref.help==1)) hw->pref.page=1;// page used for scrolling

	switch(hw->key.key){
		case '6': reverbMode=(reverbMode==0)?25:reverbMode-1; hw->sound.set_reverb(reverbMode,&ann); break;
		case '7': reverbMode=(reverbMode>=25)?0:reverbMode+1;	hw->sound.set_reverb(reverbMode,&ann);	break;

		case 'a': scroller.msg("Hello"); break;

#if defined haseffect
		case 'm': 
			effect.groundM.useDisplayList=!effect.groundM.useDisplayList;
			effect.groundM.close();
			ann.hold_multi(effect.groundM.useDisplayList,"Use Procedural Mesh","Use Mesh Display List");
			break;
#endif

		case 'z': hw->screen.set_dim(2, 2, 1,0,0,1); break; // fade to red and back ^ over 2 sec

		case '#':
			if (config.load(hw->pref.title)) ann.hold("Can't load config.");
			else                             {ann.hold("Loaded prefs from config.");
			dialog->off();
			hw->pref.help=0;
			}
			break;
	}

	if ((!hw->key.alt)&&(!hw->key.control)&&(!hw->key.shift)) { // plain special keys
		switch(hw->key.skey){
			case GLUT_KEY_PAGE_UP: case GLUT_KEY_PAGE_DOWN:
				if ((hw->pref.skipSplash)&&(hw->pref.help==0)) ann.hold("Page: %.3f", hw->pref.page);
				break;
			case GLUT_KEY_HOME: case GLUT_KEY_END:
				if ((hw->pref.skipSplash)&&(hw->pref.help==0)) ann.hold("Home: %.3f", hw->pref.home);
				break;
		}
	}

#if defined haseffect
	effect.key(&ann);
#endif

	if (hw->key.key!=0) dialog->refresh=1; // update dialog

}

//--------------------------------------------------------------------

void reset_this(void){
	log_message("Reset this...");
	hw->screen.viewport->camera.fov=90;
	hw->screen.viewport->camera.eyex=0;
	hw->screen.viewport->camera.eyey=2;
	hw->screen.viewport->camera.eyez=0;

#if defined haseffect
	effect.reset(&ann);
#endif
}

//--------------------------------------------------------------------

void splash_this(void){ // splashscreen
	float inten=.5-(fabs(frac(hw->time.time)-.5))+.5, y;  // .5->1->.5 loop

	splash.autoset=0;
	splash.set_xyz(.05*hw->screen.width,.8*hw->screen.height);
	splash.yinc=-2.25; // autostep down 2.5 char heights

	splash.bend=1;
	splash.bloom=1;
	splash.set_color(inten,inten,1-inten);
	splash.size=hw->screen.width*.035;//36
	splash.set();
	splash.print(hw->pref.title);
	splash.bend=0;
	splash.bloom=0;

	splash.set_color(1,.95,.9);
	splash.size=hw->screen.width*.015;//15;
	splash.y+=10;
	splash.set();
	splash.print("a GameDev's 'Physics' contest entry");

	splash.y+=5;
	splash.size=hw->screen.width*.008;//8;
	splash.print("Copyright (C) 2002 by George Ziniewicz - zintel.com");
	y=splash.y;

	splash.size=hw->screen.width*.0098;//10
	splash.y=.2*hw->screen.height;
	if (hw->pref.skipSplash==0) {
		splash.print("<Left-Click> to start manual camera, <Middle-Click> to stay aloft...");
		splash.print("Or press <UP/DOWN ARROW> & <PAGE UP/DOWN> to read more.");
	} else {
		splash.y+=splash.yinc;
		splash.print("Press <UP/DOWN ARROW> & <PAGE UP/DOWN> to scroll.");
	}
	splash.unset();

	listbox.x=splash.x;
	listbox.y=y-hw->screen.height*.026;//20
	listbox.size=hw->screen.width*.0098;//10
	//listbox.yinc=hw->screen.height*-.025;//19
	if (hw->screen.height>700) listbox.yinc=-19;
	else                       listbox.yinc=-15;
	listbox.draw();
}

//--------------------------------------------------------------------

void start_this(void){ // just prior to draw
	static float lastsoundtime=0;
	struct song{
		char name[12];
		float height, width;
		int freqrange,freqcenter;
	} songs[13]={
		"whoosh2811",400,200,4000,7000,//20000
		"wind2",   150,70,4000,22000,
		"birds5",   90,50,2000,11000,
		"birds6",   80,50,4000,22000,
		"birds7",   70,50,4000,22000,
		"crow",     60,50,2000,11000,
		"dohecho",  40,30,0,0,
		"wolf2",    10,50,4000,22000,
		"owl1",    -10,50,4000,11000,
		"cricket2",-30,50,6000,22000,
		"bud",     -50,50,2000,11000
	};
	int nsong=13;
	if (hw->pref.skipSplash==0) { // havent switched yet from splashscreen lookat mode?
		if (hw->mouse.button[0]) { // time to switch from lookat to manual camera?
			hw->pref.skipSplash=1;
			hw->screen.viewport->camera.place.movemode=MANUAL; // manual mode
			ann.hold("Entering manual camera mode: mouseLook + Arrow/Click move");
			hw->sound.play2d("sounds/notify8.wav");
			hw->sound.set_volume(.3);
		}
		if (hw->mouse.button[1]) {
			hw->pref.skipSplash=1;
			ann.hold("Use 'v' to change ViewCam mode, or 'r' reset");
			hw->sound.play2d("sounds/notify8.wav");
			hw->sound.set_volume(.3);
		}
	}
	if (!hw->pref.user) { // do random stuff (screen skip/splinter/hyperspace)
		int ret=randoms.update();
		if (ret==2) dialog->refresh=1;
		if ((ret==1)&&(hw->screen.viewport->camera.place.movemode!=MANUAL)) { // rnd change Option?
			static int lastoption=hw->pref.option, lastimage=hw->pref.image;
			int option=-1, image=-1;

			while ((option==-1)||(option==lastoption)) option=rndn(maxoption+1); // select new option
			lastoption=option;
			hw->pref.option=option;

#if defined haseffect
			effect.set_option(&ann);
#endif
		}
	}

	if (((hw->time.realTime-lastsoundtime)>hw->pref.minSoundInterval)&&(hw->sound.pause==0)) { // background sounds
		lastsoundtime=hw->time.realTime+(1-rndnn(2))*(hw->pref.maxSoundInterval-hw->pref.minSoundInterval);  // next sound 
		int found=-1;
		float prox=10000,p;
		float y=hw->screen.viewport->camera.place.pos.y;

		for (int i=0; i<nsong; i++) { // find closest sound
			p=fabs((y+rndn(80)-40)-songs[i].height); // get prox
			if (p<songs[i].width) {    // within its zone?
				if (p<prox) {           // closest?
					found=i;             // track it
					prox=p;
				}
			}
		}
		if (found>-1) { // found 1?
			sprintf(s,"sounds/%s.wav",songs[found].name);
			hw->sound.play(s);
			if (songs[found].freqrange) { // freq modifiable?
				hw->sound.set_frequency(rndn(songs[found].freqrange)-
					.5*songs[found].freqrange+songs[found].freqcenter);
			}
		}
	}

	if (((hw->time.realTime-lastscrolltime)>hw->pref.scrollInterval)&&(hw->pref.scrollInfo)) { // random scrolling help
		lastscrolltime=hw->time.realTime;
		switch((int)rndn(53)) {
			case 0: scroller.msg("Press 'b/B' to select BlendFunc0/BlendFunc1 for selected edit object 'e'."); break;
			case 1: scroller.msg("Press 'c' to toggle FramePerSecond Clock."); break;
			case 2: scroller.msg("Press 'C' to select ColorMapping: RGB,Sepia,Blue-Green/Green/InverseColor."); break;
			case 3: scroller.msg("Press 'd' to toggle DepthCue Fog (F5-F8 adjusts distances)."); break;
			case 4: scroller.msg("Press 'f' to toggle FullScreen/window."); break;
			case 5: scroller.msg("Press 'i' to select an Image option (texture) for selected object 'e'."); break;
			case 6: scroller.msg("Press 'I' to toggle this scrolling info."); break;
			case 7: scroller.msg("Press 'l' to select a Letterbox size 0-40% sideband."); break;
			case 8: scroller.msg("Press 'L' to toggle fullscreen lightmap (<Shift>F1-F10 for lightmap adjusters)."); break;
		case 9: scroller.msg("Press 'e' to select an object for editing (then F1-F4 texmag adjust, b/B Blending, i Image textures, and position/orient if camera not active)."); break;
			case 10: scroller.msg("Press 'n' to select a numeric info item, like camera position, render rate, sounds playing...."); break;
			case 11: scroller.msg("Press 'o' to select an Option: ground shapes."); break;
			case 12: scroller.msg("Press 'p' to toggle Pause Camera (no gravity effect)."); break;
			case 13: scroller.msg("Press 'r' to reset and randomize most params."); break;
			case 14: scroller.msg("Press 'u' to select a Universe type:starfield/particle/skybox combinations."); break;
			case 15: scroller.msg("Press 'U' to toggle User mode: no computer-randomized changes."); break;
			case 16: scroller.msg("Press 'v' to select a ViewCam mode: ManualArrow-MouseControl/Hold/LookAtSatellite/@Satellite/Orbit."); break;
			case 17: scroller.msg("Press 'V' to select a viewport for camera control (1-5 to select viewport sets)."); break;
			case 18: scroller.msg("Press 'w' to select WireFrame/Point/FilledPoly mode."); break;
			case 19: scroller.msg("Press 'x' to toggle screen splinter effect."); break;
			case 20: scroller.msg("Press '?' to view/scroll the info.txt file."); break;
			case 21: scroller.msg("Press '{}' to move camera/eye in & out."); break;
			case 22: scroller.msg("Press shift '<>' to adjust ground tesselation."); break;
			case 23: scroller.msg("Press unshifted '<>' to adjust camera FOV."); break;
			case 24: scroller.msg("Press unshifted '+-' to adjust camera rotate speed in Orbit & LookAt modes."); break;
			case 25: scroller.msg("Press shifted '+-' to adjust camera move speed in Orbit & LookAt modes."); break;

			case 26: scroller.msg("Press <PageUp>/<PageDown> to adjust ground height."); break;
			case 27: scroller.msg("Press <Delete> to Jump pure vertical (in manual camera mode)."); break;

			case 28: scroller.msg("Press F1/F2 & F3/F4 to adjust texture X & Y mag for selected object ('e')."); break;
			case 29: scroller.msg("Press F5/F6 & F7/F8 to adjust near & far fog distance."); break;

			case 30: scroller.msg("ArrowKeys move camera/selected edit object in look direction in x/z, <Control>+Arrow for x/y, <Shift>+Arrow for 10x thrust (manual camera/edit object mode ('e'))."); break;
			case 31: scroller.msg("Mouse changes camera/selected edit object orientation."); break;
			
			case 32: scroller.msg("Click Left mouse button for forward thrust, double click for 160x spurt."); break;
			case 33: scroller.msg("Click Middle mouse button for reverse thrust/brake (in manual camera mode)."); break;
			case 34: scroller.msg("Click Right mouse button to toggle popup menus and pause camera movement."); break;

			case 35: scroller.msg("LightMap adjusters, all <Shift>ed : F1/F2-texMagX  F3/F4-texMagY  F5/F6-updateInterval  F7/F8-noiseType  F9/F10-noiseLevel  F11/F12-vignetteSize"); break;
			
			case 36: scroller.msg("Note that an extensive log file is created for the current app run."); break;

			case 37: scroller.msg("Cheatcode for LookFrom Satellite mode: rvvvv (4 v's)"); break;
			case 38: scroller.msg("Cheatcode for night-vision: rCCC (3 C's)"); break;
			case 39: scroller.msg("Cheatcode for god-view: press and hold } while mouse-looking in manual camera mode."); break;
			case 40: scroller.msg("Cheatcode for freaky font terrain: reiiiiiiiiiiiiii (14 i's)."); break;
			
			case 41: scroller.msg("The yellow flashing teleporter takes you to a new ground shape of 9 types (key 'o' Options), ground texture of 4 types (key 'i' Images), and skybox universe of 12 types (key 'u' Universe)"); break;
			case 42: scroller.msg("Goal: hit the ground hard for a brain-damage effect."); break;
			case 43: scroller.msg("Goal: experience a head-spinning effect at higher elevations."); break;
			case 44: scroller.msg("Goal: fly into the rotating multi-color cube (zero gravity)."); break;
			case 45: scroller.msg("Goal: fly to and walk on the moon."); break;
			case 46: scroller.msg("Goal: stand on the moon, then run backwards (middle click) and glide off the moon."); break;
			case 47: scroller.msg("Goal: look down and double-click middle button."); break;

			case 48: scroller.msg("Newton's 1st law: An object at rest, or in motion, will remain at rest, or in motion, unless acted upon by an outside force."); break;
			case 49: scroller.msg("Newton's 2nd law: Force = Mass * Acceleration."); break;
			case 50: scroller.msg("Newton's 3rd law: For every action, there is an equal yet opposite reaction."); break;
			
			case 51: scroller.msg("PhysicZ - Copyright (C) 2002 by George Ziniewicz - zintel.com"); break;
			case 52: scroller.msg("Sh, it happens."); break;
		}
	}
	starfield2.brownian();

	if (hw->pref.universe<4) fog.set_color(0,0,0);
	else                     fog.set_color(.05,.025,0);

	if ((dialog->ID==GENERAL)&&(dialog->activeID==999)) {
		hw->pref.reset();
		reset_this();
		ann.hold("Reset to default prefs");
		dialog->off();
		dialog->activeID=-1;
	}
	if ((dialog->ID==GENERAL)&&(dialog->activeID==102)) {
		lastscrolltime=0;
		dialog->activeID=-1;
	}

#if defined haseffect
	if ((dialog->ID==LIGHTING)&&(dialog->activeID==10)) { // ambient
		effect.ground_update();
		dialog->activeID=-1;
	}
	if ((dialog->ID==COLOR)&&(dialog->activeID==39)) { // image
		hw->pref.image=dialog->activeValue;
		if (effect.dispman.selObject==0) effect.dispman.dos[                         0].vis.image=hw->pref.image;
		else                             effect.dispman.dos[effect.dispman.selObject-1].vis.image=hw->pref.image;
		dialog->activeID=-1;
	}
	if ((dialog->ID==THIS)&&(dialog->activeID==105)) { // edit obj annot
		if (effect.dispman.selObject<9)
			ann.hold_multi(effect.dispman.selObject,
			"No Object selected",
			"Edit Ground","Edit Ground Rough Color","Edit Ground Detail","Edit Ground Shadow",
			"Edit Cloud 1","Edit Cloud 2",
			"Edit Satellite","Edit Water");
		else ann.hold_multi(effect.dispman.selObject-9,"Edit Teleporter","Edit Cube","Edit Moon");
		effect.object2pref();
		dialog->activeID=-1;
	}

	effect.start_frame(&ann);
#endif
}

//--------------------------------------------------------------------

void end_this(void){
	
#if defined haseffect
	effect.end_frame(&ann);
#endif
}

//--------------------------------------------------------------------
