//-------------------------------------------------------------------
//	File:		   sound.c
//	Comments:   by George Ziniewicz
//-------------------------------------------------------------------

#include "sound.h"
#include "text.h"
#include "utility.h"
#include "hardware.h"

//--------------------------------------------------------------------------------------

Sound::Sound() {
// int flags
	hasEAX=0;
	hasHw=0;
	inited=0;
	lastChannel=-1;
	numDrivers=0;
	on=0;
	pause=0;
	reverbMode=0;
	sampleChannel=-1;
	soundDriver=0; // bug: default to 1st driver if cant find one
	soundOutput=-1;
	type=FSOUND_2D;
	verbosity=1;

// vars
	mindistance=100;
	maxdistance=1000;
	rolloff=1;
	volume_master=1;

// pointers
	hw=0;
	sample=0;
	sampletmp=0;
};

	//volume_music=1;
	//pan=.5;
	//on_music=0;
	//glist=0;
	//material=0;

Sound::~Sound() {};

//----------------------------------------------------------------------------------
//----------------------------------------------------------------------------------
//----------------------------------------------------------------------------------

int Sound::init(void) {
	int i;
	unsigned int caps;
	signed char *driverName=new signed char[80];

	hw=Hw::get();

	log_message("Init Sound...");

	sprintf(s,"   FMOD version: %.02f", FSOUND_GetVersion()); log_message(s);
	if (FSOUND_GetVersion()< FMOD_VERSION) { // check version
		sprintf(s,"   Incorrect/outdated library version, should be FMOD %.02f **********", FMOD_VERSION);
		log_message(s);
		return 1;
	}
	log_message("   Sound: setting output to FSOUND_OUTPUT_DSOUND...");
	FSOUND_SetOutput(FSOUND_OUTPUT_DSOUND); // set output
	soundOutput=FSOUND_GetOutput();
	if (soundOutput!=FSOUND_OUTPUT_DSOUND) {log_message("   Sound: cannot set to DirectSound **********"); return 2;}
	
	numDrivers=FSOUND_GetNumDrivers(); // check drivers
	sprintf(s,"   # Drivers: %d", numDrivers); log_message(s);
	if (numDrivers==0) { log_message("   Sound: no sound drivers error **********"); return 3;}
	soundDriver=numDrivers-1; // use highest driver assuming newer is better

	log_message(" ");
	log_message("   Driver#  Name                  Capabilities");
	for (i=0; i<numDrivers; i++) { // check capabilities
		char capss[80]={0};

		caps=0;
		FSOUND_GetDriverCaps(i, &caps);
		if (caps & FSOUND_CAPS_HARDWARE) {hasHw=1;  strcpy(capss,"HARDWARE");}
		if (caps & FSOUND_CAPS_EAX2) 		{
			hasEAX=1;
			if (capss[0]!=0) strcat(capss," + ");
			strcat(capss,"EAX");
		}
		driverName=FSOUND_GetDriverName(FSOUND_GetDriver());
		sprintf(s,  "     %1d      %20s  %s", i, driverName, capss);
		log_message(s);
	}
	log_message(" ");
	if ((hw->pref.useSoundDriver>=0)&&(hw->pref.useSoundDriver<numDrivers)) {
		soundDriver=hw->pref.useSoundDriver;
		log_message("   Sound: Setting driver by config directive.");
	}
	sprintf(s,"   Using Sound Driver #: %d", soundDriver); log_message(s);
	if (FSOUND_SetDriver(soundDriver)==0) { log_message("   Sound: Cannot set using driver **********"); return 4;}
	
	log_message("   Sound: setting mixer...");
	if (FSOUND_SetMixer(FSOUND_MIXER_QUALITY_AUTODETECT)==0) {
		log_message("   Sound: Cannot autoset mixer **********");
		return 5;
	}

	log_message("   Sound: final sound init...");
	if (FSOUND_Init(44000, 32, 0)==0) {log_message("   Sound: bad initilization:"); log_error(); return 6;}
	inited=1; on=1; //on_music=1;
	set_master_volume(1);
	if (hasHw) {
		log_message("   Sound: all ok, setting 3d params...");
		type=FSOUND_HW3D;
		FSOUND_3D_Listener_SetDistanceFactor(hw->pref.meter);//how many units in a meter
		FSOUND_3D_Listener_SetRolloffFactor(rolloff);
		//open_geom();
	}
	log_message("   Sound: ready.");
	return 0;
}

//----------------------------------------------------------------------------------

void Sound::close(void) {
	if (!inited) return;
	free_sample(sample);
	free_sample(sampletmp);
	close_geom();
	FSOUND_Close();
	inited=0;
	return;
}

//----------------------------------------------------------------------------------

int Sound::play(const char * ins) { // play a wav file
	if (!on) {
		//log_message("Sound not On"); 
		return -1;
	}
	if (hasHw) return play(ins,hw->screen.viewport->camera.place.pos.x,
		                        hw->screen.viewport->camera.place.pos.y,
										hw->screen.viewport->camera.place.pos.z);
	if (verbosity>1) {sprintf(s,"Playing: %s...", ins); log_message(s);}
	sampletmp   =FSOUND_Sample_Load(FSOUND_FREE,ins,FSOUND_2D,0);
	if (sampletmp==0)	{
		sprintf(s,"Can't load sampletmp: %s as 2d **********", ins); log_message(s);
		sampletmp=FSOUND_Sample_Load(FSOUND_FREE,ins,FSOUND_STEREO,0);
	}
	if (sampletmp==0) {
		sprintf(s,"Can't load sampletmp: %s as stereo **********", ins); log_message(s); return -1;
	}
	return finish_2d(sampletmp);
}

//-------------------------------------------------------------------------

int Sound::play2d(const char * ins) { // play a wav file
	if (!on) {
		//log_message("Sound not On");
		return -1;
	}
	if (verbosity>1) {sprintf(s,"Playing2d: %s...", ins); log_message(s);}
	sampletmp   =FSOUND_Sample_Load(FSOUND_FREE,ins,FSOUND_2D,0);
	if (sampletmp==0)	{
		sprintf(s,"Can't load sampletmp: %s as 2d **********", ins); log_message(s);
		sampletmp=FSOUND_Sample_Load(FSOUND_FREE,ins,FSOUND_STEREO,0);
	}
	if (sampletmp==0) {
		sprintf(s,"Can't load sampletmp: %s as stereo **********", ins); log_message(s);	
		sampletmp   =FSOUND_Sample_Load(FSOUND_FREE,ins,FSOUND_MONO,0);
	}
	if (sampletmp==0) {
		sprintf(s,"Can't load sampletmp: %s as mono **********", ins); log_message(s);	
		return -1;
	}
	return finish_2d(sampletmp);
}

//-------------------------------------------------------------------------

int Sound::play_type(const char * ins, int type) { // play a wav file
	if (!on) {
		//log_message("Sound not On");
		return -1;
	}
	if (verbosity>1) {sprintf(s,"Playing2d: %s...", ins); log_message(s);}
	sampletmp   =FSOUND_Sample_Load(FSOUND_FREE,ins,type,0);
	if (sampletmp==0)	{
		sprintf(s,"Can't load sampletmp: %s as requested type **********", ins); log_message(s);
		sampletmp   =FSOUND_Sample_Load(FSOUND_FREE,ins,FSOUND_2D,0);
	}
	if (sampletmp==0)	{
		sprintf(s,"Can't load sampletmp: %s as 2d **********", ins); log_message(s);
		sampletmp=FSOUND_Sample_Load(FSOUND_FREE,ins,FSOUND_STEREO,0);
	}
	if (sampletmp==0) {
		sprintf(s,"Can't load sampletmp: %s as stereo **********", ins); log_message(s);	
		sampletmp   =FSOUND_Sample_Load(FSOUND_FREE,ins,FSOUND_MONO,0);
	}
	if (sampletmp==0) {
		sprintf(s,"Can't load sampletmp: %s as mono **********", ins); log_message(s);	
		return -1;
	}
	return finish_2d(sampletmp);
}

//-------------------------------------------------------------------------

int Sound::play(FSOUND_SAMPLE *sample) { // play a sample
	if (!on) {
		//log_message("Sound not On");
		return -1;
	}
	if (sample==0) {log_message("Can't play 2d sample==0 **********"); return -1;}
	if (verbosity>1) {sprintf(s,"Playing Sample#: %d...", sample); log_message(s);}
	if (hasHw) return play(sample,hw->screen.viewport->camera.place.pos.x,
		                           hw->screen.viewport->camera.place.pos.y,
											hw->screen.viewport->camera.place.pos.z);
	return finish_2d(sample);
}

//-------------------------------------------------------------------------

int Sound::finish_2d(FSOUND_SAMPLE *sample) {
	if (verbosity>2) {sprintf(s,"Finishing2d Sample: %d...", sample); log_message(s);}
	lastChannel=FSOUND_PlaySound(FSOUND_FREE, sample);
	if (lastChannel<0) {log_message("Can't play sample in finish2d **********"); return -1;}
	hw->pref.lastsoundtime=hw->time.realTime;
	return 0;
}

//-------------------------------------------------------------------------

void Sound::set_master_volume(float involume) {
	if (!inited) return;
	volume_master=involume;
	FSOUND_SetSFXMasterVolume(involume*255);
	return;
}

//-------------------------------------------------------------------------

void Sound::set_volume(float involume) {
	if (!inited) return;
	 if (lastChannel>=0) FSOUND_SetVolume(lastChannel,involume*255);
	return;
}

//-------------------------------------------------------------------------

float Sound::get_volume(int channel){
	if (channel>=0) return FSOUND_GetVolume(channel)/255.;
	else            return 1;
}

//-------------------------------------------------------------------------

void Sound::set_pan(float inpan) {
	if (!inited) return;
	if (lastChannel>=0) FSOUND_SetPan(lastChannel,inpan*255);
	return;
}

//-------------------------------------------------------------------------

void Sound::set_frequency(int infrequency) {
	if (!inited) return;
	if (lastChannel>=0) FSOUND_SetFrequency(lastChannel,infrequency);
	return;
}

//-------------------------------------------------------------------------

int Sound::get_frequency(int channel) {
	if (!inited) return 0;
	if (channel>=0) return FSOUND_GetFrequency(channel);
	else return 0;
}

//-------------------------------------------------------------------------

void Sound::set_reserved(void) {
	FSOUND_SetReserved(lastChannel,TRUE);
}

//-------------------------------------------------------------------------

void Sound::set_surround(void) {
	FSOUND_SetSurround(lastChannel,TRUE);
}

//-------------------------------------------------------------------------

void Sound::set_rolloff(float value) {
	if (!inited) return;
	rolloff=value;
	FSOUND_3D_Listener_SetRolloffFactor(rolloff);
}

//-------------------------------------------------------------------------

void Sound::free_sample(FSOUND_SAMPLE *sample) {
	if (sample) FSOUND_Sample_Free(sample);
}

//-------------------------------------------------------------------------


FSOUND_SAMPLE *Sound::load_wav(const char *ins) {
	FSOUND_SAMPLE *samplei=0;

	if (on) {
		if (verbosity>1) {sprintf(s,"LoadingSample: %s...", ins); log_message(s);}
		samplei   =FSOUND_Sample_Load(FSOUND_FREE, ins, type, 0); // attempt to load as default
		if (samplei==0) {
			samplei=FSOUND_Sample_Load(FSOUND_FREE, ins, FSOUND_STEREO, 0); // try load as stereo
			if (samplei==0) {
				sprintf(s,"Cannot load sample: %s **********",ins); log_message(s);
			}
		}
	}
	return samplei;
}


//-------------------------------------------------------------------------
//-------------------------------------------------------------------------
//----------------------------------------------------- 3d ----------------

int Sound::play(const char * ins, float x, float y, float z) { // play a wav file in 3d
	if (!on) {log_message("Sound not On"); return -1;}
	if (!hasHw) return play(ins);
	if (verbosity>1) {sprintf(s,"Playing3d: %s...", ins); log_message(s);}
	sampletmp      =FSOUND_Sample_Load(FSOUND_FREE,ins,FSOUND_HW3D,0);
	if (sampletmp==0) {
		sprintf(s,"Can't load sampletmp: %s as 3d **********", ins); log_message(s);
	   sampletmp   =FSOUND_Sample_Load(FSOUND_FREE,ins,FSOUND_2D,0);
	}
	if (sampletmp==0) {
		sprintf(s,"Can't load sampletmp: %s as 2d **********", ins); log_message(s);
		sampletmp   =FSOUND_Sample_Load(FSOUND_FREE,ins,FSOUND_STEREO,0);
	}
	if (sampletmp==0) {
		sprintf(s,"Can't load sampletmp: %s as stereo **********", ins); log_message(s);
		sampletmp   =FSOUND_Sample_Load(FSOUND_FREE,ins,FSOUND_MONO,0);
	}
	if (sampletmp==0) {
		sprintf(s,"Can't load sampletmp: %s as mono **********", ins); log_message(s);
		return -1;
	}
	return finish_3d(sampletmp,x,y,z);
}

//-------------------------------------------------------------------------

int Sound::play(FSOUND_SAMPLE *sample, float x, float y, float z) { // play a sample in 3d
	if (!on) {
		//log_message("Sound not On");
		return -1;
	}
	if (sample==0) {log_message("Can't play 3d sample==0 **********"); return -1;}
	if (verbosity>1) {sprintf(s,"Playing Sample3d#: %d...", sample); log_message(s);}
	if (hasHw) return finish_3d(sample,x,y,z);
	else       return play(sample);
	
}

//-------------------------------------------------------------------------

int Sound::finish_3d(FSOUND_SAMPLE *sample, float inx, float iny, float inz) {
	float pos[]={inx,iny,-inz},vel[]={0,0,0};//-z

	FSOUND_Sample_SetMinMaxDistance(sample, mindistance, maxdistance);// set sample volume range
	if (verbosity>2) {sprintf(s,"Finishing3d Sample: %d...", sample); log_message(s);}
	lastChannel=FSOUND_PlaySound(FSOUND_FREE, sample);  // play
	if (lastChannel<0) {log_message("Can't play sample in finish3d **********"); return -1;}
	FSOUND_3D_SetAttributes(lastChannel,pos,vel);// set channel attr
	//FSOUND_Reverb_SetProperties(lastChannel, FSOUND_REVERBMIX_USEDISTANCE);//set channel reverb
	hw->pref.lastsoundtime=hw->time.realTime;
	return 0;
}

//-------------------------------------------------------------------------

void Sound::set_position(int channel, float inx, float iny, float inz) {
	float pos[]={inx,iny,-inz},vel[]={0,0,0}; //-z

	if (hasHw) FSOUND_3D_SetAttributes(channel,pos,vel);// set channel attr
}

//-------------------------------------------------------------------------

void Sound::update_listener(void) {
	float pos[]={hw->screen.viewport->camera.place.pos.x,
		          hw->screen.viewport->camera.place.pos.y,
				   -hw->screen.viewport->camera.place.pos.z},vel[]={0,0,0};//-z

	if (!hasHw) return;// no 3d?
	FSOUND_3D_Listener_SetAttributes(pos, vel,
		 hw->screen.viewport->camera.model[8],   //camera vectors
		 hw->screen.viewport->camera.model[9],
		 hw->screen.viewport->camera.model[10],
		 hw->screen.viewport->camera.model[4],
		 hw->screen.viewport->camera.model[5],
		 hw->screen.viewport->camera.model[6]);
	FSOUND_3D_Update();
}

//---------------------------------------------------------------------------

void Sound::update_listener(float x, float y, float z,
									 float fx,float fy,float fz,
									 float ux,float uy,float uz) {
	float pos[]={x,y,-z},vel[]={0,0,0};//-z

	if (!hasHw) return;// no 3d?
	FSOUND_3D_Listener_SetAttributes(pos, vel, fx,fy,fz, ux,uy,uz);
	FSOUND_3D_Update();
}

//---------------------------------------------------------------------------

void Sound::stop_sound(void) {
	if (inited) FSOUND_StopSound(FSOUND_ALL);
}

//-------------------------------------------------------------------------------

void Sound::set_pause(int inpause) {
	pause=inpause;
	if (on) FSOUND_SetPaused(FSOUND_ALL,pause);
}

//-------------------------------------------------------------------------------

int Sound::get_channels_playing(void) {
	if (on) return FSOUND_GetChannelsPlaying();
	else    return 0;
}

//-------------------------------------------------------------------------------

void Sound::set_loop(FSOUND_SAMPLE *sample) {
	if (on) FSOUND_Sample_SetLoopMode(sample,FSOUND_LOOP_NORMAL);
}

//-------------------------------------------------------------------------------

void Sound::log_error(void) {
	char s[80];
	sprintf(s,"%s **********",FMOD_ErrorString(FSOUND_GetError()));
	log_message(s);
}

//-------------------------------------------------------------------------------
//-------------------------------------------------------------------------------
//-------------------------------------------------------------------------------

int Sound::open_geom(void) {
	if (!hasHw) return 1;// no 3d?
	if (glist) FSOUND_Geometry_List_Free(glist);//if used, reset
	glist=FSOUND_Geometry_List_Create(TRUE); // new geom list
	if (glist==0)	{ // err?
		log_message("Sound: cannot create geom list **********");
		return 1;
	}
	FSOUND_Geometry_List_Begin(glist); // begin list
		if (material) FSOUND_Geometry_Material_Free(material);// if used, reset
		material = FSOUND_Geometry_Material_Create();         // new material
		FSOUND_Geometry_Material_SetAttributes(material, 1, 1, 0, 0); // set material
		FSOUND_Geometry_Material_Set(material);// update material
		make_geom_polys();               // get poly list
		FSOUND_Geometry_List_End(glist); // end list
	FSOUND_Geometry_AddList(glist);
	return 0;
}

//-------------------------------------------------------------------------

// COORDINATE SYSTEM : X = right, Y = up, Z = forwards. (Left handed)
void Sound::make_geom_polys(void) {
	int i,j,k;
	float size=hw->pref.worldSize*2;
	float poly[5][4][3]={
			-1,-1,-1, -1,-1, 1,	-1, 1, 1, -1, 1,-1,// left wall
			 1,-1, 1,  1,-1,-1,	 1, 1,-1,  1, 1, 1,// right
			-1,-1, 1,  1,-1, 1,	 1, 1, 1, -1, 1, 1,// front
			 1,-1,-1, -1,-1,-1, -1, 1,-1,  1, 1,-1,// rear
			-1, 1, 1,  1, 1, 1,  1, 1,-1, -1, 1,-1};// top
	float normal[5][3]={
			 1, 0, 0, // left wall
			-1, 0, 0, // right wall
			 0, 0,-1, // front wall
			 0, 0, 1, // back wall
			 0,-1, 0};// top

	if (!hasHw) return;
	for         (i=0; i<5; i++) {
		for     (j=0; j<4; j++) {
			for (k=0; k<3; k++) poly[i][j][k]*=size;
		}
	}
	for (i=0; i<4; i++) {
		FSOUND_Geometry_AddPolygon(poly[i][0], poly[i][1],
								   poly[i][2], poly[i][3],
								   NULL, FSOUND_GEOMETRY_NORMAL |
								   FSOUND_GEOMETRY_REFLECTIVE, 0);
	}
	return;
}

//---------------------------------------------------------------------------

	//FSOUND_Geometry_Material_SetAttributes(material, 1, 1, 0.1f, 0.1f);
	//FSOUND_Geometry_Material_SetAttributes(material,0,0, 0,0);

//-------------------------------------------------------------------------

void Sound::close_geom(void) {
	if (!hasHw) return;// no 3d?
	if (glist)    FSOUND_Geometry_List_Free(glist);
	if (material) FSOUND_Geometry_Material_Free(material);
	return;
}
		
//---------------------------------------------------------------------------

void Sound::set_reverb(int envn, Text *ann) {
	if (!hasHw) return; // no 3d?
	reverbMode=envn;
	if (envn==0) {FSOUND_REVERB_PROPERTIES props=FSOUND_PRESET_GENERIC;          FSOUND_Reverb_SetProperties(&props); ann->hold("GENERIC"); }
	if (envn==1) {FSOUND_REVERB_PROPERTIES props=FSOUND_PRESET_PADDEDCELL;       FSOUND_Reverb_SetProperties(&props); ann->hold("PADDEDCELL"); }
	if (envn==2) {FSOUND_REVERB_PROPERTIES props=FSOUND_PRESET_ROOM;             FSOUND_Reverb_SetProperties(&props); ann->hold("ROOM"); }
	if (envn==3) {FSOUND_REVERB_PROPERTIES props=FSOUND_PRESET_BATHROOM;         FSOUND_Reverb_SetProperties(&props); ann->hold("BATHROOM"); }
	if (envn==4) {FSOUND_REVERB_PROPERTIES props=FSOUND_PRESET_LIVINGROOM;       FSOUND_Reverb_SetProperties(&props); ann->hold("LIVINGROOM"); }
	if (envn==5) {FSOUND_REVERB_PROPERTIES props=FSOUND_PRESET_STONEROOM;        FSOUND_Reverb_SetProperties(&props); ann->hold("STONEROOM"); }
	if (envn==6) {FSOUND_REVERB_PROPERTIES props=FSOUND_PRESET_AUDITORIUM;       FSOUND_Reverb_SetProperties(&props); ann->hold("AUDITORIUM"); }
	if (envn==7) {FSOUND_REVERB_PROPERTIES props=FSOUND_PRESET_CONCERTHALL;      FSOUND_Reverb_SetProperties(&props); ann->hold("CONCERTHALL"); }
	if (envn==8) {FSOUND_REVERB_PROPERTIES props=FSOUND_PRESET_CAVE;             FSOUND_Reverb_SetProperties(&props); ann->hold("CAVE"); }
	if (envn==9) {FSOUND_REVERB_PROPERTIES props=FSOUND_PRESET_ARENA;            FSOUND_Reverb_SetProperties(&props); ann->hold("ARENA"); }
	if (envn==10) {FSOUND_REVERB_PROPERTIES props=FSOUND_PRESET_HANGAR;          FSOUND_Reverb_SetProperties(&props); ann->hold("HANGAR"); }
	if (envn==11) {FSOUND_REVERB_PROPERTIES props=FSOUND_PRESET_CARPETTEDHALLWAY;FSOUND_Reverb_SetProperties(&props); ann->hold("CARPETTEDHALLWAY"); }
	if (envn==12) {FSOUND_REVERB_PROPERTIES props=FSOUND_PRESET_HALLWAY;         FSOUND_Reverb_SetProperties(&props); ann->hold("HALLWAY"); }
	if (envn==13) {FSOUND_REVERB_PROPERTIES props=FSOUND_PRESET_STONECORRIDOR;   FSOUND_Reverb_SetProperties(&props); ann->hold("STONECORRIDOR"); }
	if (envn==14) {FSOUND_REVERB_PROPERTIES props=FSOUND_PRESET_ALLEY;           FSOUND_Reverb_SetProperties(&props); ann->hold("ALLEY"); }
	if (envn==15) {FSOUND_REVERB_PROPERTIES props=FSOUND_PRESET_FOREST;          FSOUND_Reverb_SetProperties(&props); ann->hold("FOREST"); }
	if (envn==16) {FSOUND_REVERB_PROPERTIES props=FSOUND_PRESET_CITY;            FSOUND_Reverb_SetProperties(&props); ann->hold("CITY"); }
	if (envn==17) {FSOUND_REVERB_PROPERTIES props=FSOUND_PRESET_MOUNTAINS;       FSOUND_Reverb_SetProperties(&props); ann->hold("MOUNTAINS"); }
	if (envn==18) {FSOUND_REVERB_PROPERTIES props=FSOUND_PRESET_QUARRY;          FSOUND_Reverb_SetProperties(&props); ann->hold("QUARRY"); }
	if (envn==19) {FSOUND_REVERB_PROPERTIES props=FSOUND_PRESET_PLAIN;           FSOUND_Reverb_SetProperties(&props); ann->hold("PLAIN"); }
	if (envn==20) {FSOUND_REVERB_PROPERTIES props=FSOUND_PRESET_PARKINGLOT;      FSOUND_Reverb_SetProperties(&props); ann->hold("PARKINGLOT"); }
	if (envn==21) {FSOUND_REVERB_PROPERTIES props=FSOUND_PRESET_SEWERPIPE;       FSOUND_Reverb_SetProperties(&props); ann->hold("SEWERPIPE"); }
	if (envn==22) {FSOUND_REVERB_PROPERTIES props=FSOUND_PRESET_UNDERWATER;      FSOUND_Reverb_SetProperties(&props); ann->hold("UNDERWATER"); }
	if (envn==23) {FSOUND_REVERB_PROPERTIES props=FSOUND_PRESET_DRUGGED;         FSOUND_Reverb_SetProperties(&props); ann->hold("DRUGGED"); }
	if (envn==24) {FSOUND_REVERB_PROPERTIES props=FSOUND_PRESET_DIZZY;           FSOUND_Reverb_SetProperties(&props); ann->hold("DIZZY"); }
	if (envn==25) {FSOUND_REVERB_PROPERTIES props=FSOUND_PRESET_PSYCHOTIC;       FSOUND_Reverb_SetProperties(&props); ann->hold("PSYCHOTIC"); }
}

//-------------------------------------------------------------------------------
//-------------------------------------------------------------------------------
//-------------------------------------------------------------------------------
/*
FMUSIC_MODULE *Sound::load_song(char *s) {
	FMUSIC_MODULE *mod;

	if (on_music) mod=FMUSIC_LoadSong(s);
	return mod;
}

//-------------------------------------------------------------------------------

void Sound::play_music(FMUSIC_MODULE *mod) {
	if ((mod)&&(on_music)) FMUSIC_PlaySong(mod);
}

//-------------------------------------------------------------------------------

void Sound::stop_music(FMUSIC_MODULE *mod) {
	if (mod) FMUSIC_StopSong(mod);
}

//-------------------------------------------------------------------------------

void Sound::set_music_volume(FMUSIC_MODULE *mod, float value) {
	volume_music=value;
	if (mod) FMUSIC_SetMasterVolume(mod,value*255);
}

//-------------------------------------------------------------------------------

void Sound::free_song(FMUSIC_MODULE *mod) {
	if (mod) FMUSIC_FreeSong(mod);
}

//-------------------------------------------------------------------------------

int Sound::music_finished(FMUSIC_MODULE *mod) {
	if (mod) return FMUSIC_IsFinished(mod);
	else return 1;
}

//-------------------------------------------------------------------------------
*/


