//-------------------------------------------------------------------
//	File:		   texture.cpp
//	Comments:   by George Ziniewicz
//-------------------------------------------------------------------

#include "texture.h"
#include "utility.h"
#include "hardware.h"

//-------------------------------------------------------------------

Texture::Texture() {
	texID=0;
	repeatMode=GL_REPEAT;
	blendMode =GL_MODULATE;
	type=GL_RGBA;//GL_LUMINANCE_ALPHA
	smokeiteration=64;
	smokesize=16;
	sizex=128;
	sizey=128;
	envr=envg=envb=1; enva=1;
	lastUpdateTime=0;
	for (int i=0; i<19; i++) {
		textures[i]=0;
		names[i][0]=0;
	}
};

Texture::~Texture() {
	if (texID) glDeleteTextures(1, &texID);
};

GLuint Texture::lastTexID=0;
float  Texture::envr=1;
float  Texture::envg=1;
float  Texture::envb=1;
float  Texture::enva=1;
GLuint Texture::globalBlendMode=0;
int    Texture::ntextures=0;
Texture *Texture::textures[19];
char   Texture::names[19][80];

//--------------------------------------------------------------------

void Texture::close(void) {
	char s[80];

	for (int i=0; i<ntextures; i++) {
		if (textures[i]!=0) {
			if (textures[i]->texID!=0) {
				sprintf(s,"   Destroying texture ID %u...",textures[i]->texID);
				log_message(s);
				glDeleteTextures(1, &textures[i]->texID);
			}
		}
	}
}

//--------------------------------------------------------------------

void Texture::bind(void) {

	if (texID==0) glGenTextures(1, &texID);
	if (texID==lastTexID) return; // avoid rebinding current texture again
	lastTexID=texID;
	glBindTexture(GL_TEXTURE_2D, texID);
	if (repeatMode) { // if custom repeatmode
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,repeatMode);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,repeatMode);
	}
	if (globalBlendMode) {
		glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,globalBlendMode); // if custom blend
		if (globalBlendMode==GL_BLEND) {
			float color[4]={envr,envg,envb,enva};
			glTexEnvfv(GL_TEXTURE_ENV,GL_TEXTURE_ENV_COLOR,color);
		}
	} else if (blendMode) {
		glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,blendMode);
		if (blendMode==GL_BLEND) {
			float color[4]={envr,envg,envb,enva};
			glTexEnvfv(GL_TEXTURE_ENV,GL_TEXTURE_ENV_COLOR,color);
		}
	}
	return;
}

//--------------------------------------------------------------------

void Texture::mip(int mipmap) {
   glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
	if (mipmap) {
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST_MIPMAP_NEAREST);
	} else {
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
	}
	return;
}

//----------------------------------------------------------------------------------------

int Texture::load_tex(const char *picname, GLenum textype, int alpha, int threshold, int mipmap) {
	FILE *file;
	unsigned char array[262144]; // up to 256 sqr
	int x,y, addr, red,green,blue, alphaindex;
	GLuint ttype;
	char s[80];

	file= fopen(picname, "rb");
	if (file == 0) return 1; // no file
	fseek(file, 18, SEEK_SET);
	sizex=fgetc(file)+fgetc(file)*256;
	fseek(file, 22, SEEK_SET);
	sizey=fgetc(file)+fgetc(file)*256;
	if (sizex*sizey>65536) return 2; // too big

	textures[ntextures]=this;
	strcpy(&names[ntextures][0],picname);
	fseek(file, 54, SEEK_SET);
	type=textype;

	for    (y = 0; y <sizey; y++) {
		for (x = 0; x <sizex; x++) {
			blue =fgetc(file);
			green=fgetc(file);
			red  =fgetc(file);

			switch(type) {
				case GL_RGBA:
					addr=(x+y*sizey)*4;
					alphaindex=3;
					array[addr  ] = red;
					array[addr+1] = green;
					array[addr+2] = blue;
					break;
				case GL_LUMINANCE_ALPHA:
					addr=(x+y*sizey)*2;
					array[addr] = red;
					alphaindex=1;
					break;
			}
			if (alpha<0) array[addr+alphaindex]=(red+green+blue)/3; // "-" alpha flag means compute alpha from color
			else {
				if ((red >=threshold) ||                   // else set texel alpha to this alpha if r|g|b >=threshold
					(green>=threshold) ||
					(blue >=threshold)) array[addr+alphaindex]=alpha;
				else				        array[addr+alphaindex]=0;
			}
		}
	}
	fclose(file);
   bind();
	sprintf(s,"   Load texture #%2d  ID:%2u  Name: %s",ntextures++,texID,picname);
	log_message(s);
	mip(mipmap);
	//if (type==GL_RGBA) ttype=GL_RGBA8;
	//else
	ttype=type;
	glTexImage2D(                 GL_TEXTURE_2D,0,ttype,sizex,sizey,0,type,GL_UNSIGNED_BYTE,array);
	if (mipmap) gluBuild2DMipmaps(GL_TEXTURE_2D,4,      sizex,sizey,  type,GL_UNSIGNED_BYTE,array);
	return 0;
}

//---------------------------------------------------------------------------

int Texture::make_tex(int proctype, int insizex, int insizey, GLenum textype, int mipmap) {
	unsigned char array[262144];
	float f[256][256];
	int i, addr, x,y;
	float r,g,b,lum, len;
	GLuint ttype;
	char s[80];

	sizex=insizex;
	sizey=insizey;
	type=textype;
	if ((sizex*sizey)>65536) return 1;
	textures[ntextures]=this;
	switch(proctype) {
		case SMOKE:     strcpy(&names[ntextures][0],"SMOKE"); break;
		case SPHERE:    strcpy(&names[ntextures][0],"SPHERE"); break;
		case NOTSPHERE: strcpy(&names[ntextures][0],"NOTSPHERE"); break;
		case GRADIENT:  strcpy(&names[ntextures][0],"GRADIENT"); break;
		case HALO:      strcpy(&names[ntextures][0],"HALO"); break;
		case SAND:      strcpy(&names[ntextures][0],"SAND"); break;
		case STAR:      strcpy(&names[ntextures][0],"STAR"); break;
	}

	if (proctype==SMOKE) {// smoke multipass proc ---------------------
		float min=99999, max=-1;
		for (i=0; i<smokeiteration; i++) { // for many little spheres
			int rx, ry;

			rx=(rndn(1))*rndn(sizex-smokesize*2)+smokesize; // pick a rnd location for a small sphere //sqrt for offcenter
			ry=(rndn(1))*rndn(sizey-smokesize*2)+smokesize;
			for    (y = 0; y <sizey; y++) {
				for (x = 0; x <sizex; x++) {
					if (i==0) f[y][x]=0;
					lum=sqrt((x-rx)*(x-rx)+(y-ry)*(y-ry))/smokesize; // make a small sphere
					if (lum>1) lum=1;
					lum=1-lum;
					f[y][x]+=lum; // impress it into texture
					if (f[y][x]>max) max=f[y][x]; // track max pixel inten for scaling later
				}
			}
		}
		max=(255./max);
		for    (y = 0; y <sizey; y++) { // scale smoke pixel to 0->255
			for (x = 0; x <sizex; x++) {
				lum=f[y][x]*max;
				switch(type) {
					case GL_RGBA:
						addr=(x+y*sizey)*4;
						array[addr  ] = lum;
						array[addr+1] = lum;
						array[addr+2] = lum;
						array[addr+3] = lum;
						break;
					case GL_LUMINANCE_ALPHA:
						addr=(x+y*sizey)*2;
						array[addr]   = lum;
						array[addr+1] = lum;
						break;
				}
			}
		}
	}
	else { //  1 pass proc --------------------------------------
		float x1,y1;
		int sizexh=sizex*.5, sizeyh=sizey*.5;
		for    (y = 0; y <sizey; y++) {
			y1=y-sizeyh;
			for (x = 0; x <sizex; x++) {
				x1=x-sizexh;
				switch(proctype) {
				case SPHERE:        // sphere
					lum=sqrt(x1*x1+y1*y1)/sizexh;
					if (lum>1) lum=1;
					lum=lum*lum;
					lum=(1-lum)*255;
					r=g=b=lum;
					break;
				case NOTSPHERE:        // inv sphere
					lum=sqrt(x1*x1+y1*y1)/sizexh;
					if (lum>1) lum=1;
					lum=lum*lum;
					lum*=255;
					r=g=b=lum;
					break;
				case GRADIENT:        // gradient
					lum=(16.-sqrt(y*2.))*15;
					r=g=b=lum;
					break;
				case HALO:        // halo
					len=sqrt((x-sizexh)*(x-sizexh)+(y-sizeyh)*(y-sizeyh));
					lum=len/58;
					if (lum>1) lum=9-lum*8;
					if (lum<0) lum=0;
					else lum=pow(lum,1)*255;
					r=g=b=lum;
					break;
				case SAND: { // grit
						float n=rndn(1);
						lum=255;
						if      (n<.001) r=g=b=0;
						else if (n<.002) r=g=b=255;
						else {
							r=rndn(127)+127;
							g=r*.8;
							b=r*.6;
							lum=r;
						}
					}
					break;
				case STAR:       // star
					float lx,ly;
					lx=(sizexh-fabs(x-sizexh))/sizexh;
					ly=(sizeyh-fabs(y-sizexh))/sizexh;
					lum=lx*ly;
					lum= (lum<0) ? 0 : (lum>1) ? 1 : lum;
					lum*=255;
					r=g=b=lum;
					break;
				}
				switch(type) {
					case GL_RGBA:
						addr=(x+y*sizey)*4;
						array[addr  ] = r;
						array[addr+1] = g;
						array[addr+2] = b;
						array[addr+3] = lum;
						break;
					case GL_LUMINANCE_ALPHA:
						addr=(x+y*sizey)*2;
						array[addr  ] = r;
						array[addr+1] = lum;
						break;
				}
			}
		}
	}
   bind();
	sprintf(s,"   Make texture #%2d  ID:%2u  Name: %s",ntextures,texID,&names[ntextures++][0]);
	log_message(s);
	mip(mipmap);
	//if (type==GL_RGBA) ttype=GL_RGBA8;
	//else
	ttype=type;
	glTexImage2D(                 GL_TEXTURE_2D,0,ttype,sizex,sizey,0,type,GL_UNSIGNED_BYTE,array);
	if (mipmap) gluBuild2DMipmaps(GL_TEXTURE_2D,4      ,sizex,sizey,  type,GL_UNSIGNED_BYTE,array);
	return 0;
}

//---------------------------------------------------------------------------------------

void Texture::lightmap(int intype, int grain, int insize) { // make multipattern mask
	unsigned char array[262144];
	int xnoisev[256];
	int i,addy,addr, x,y, ynoise, sizeh, grainh=grain*.5;
	float r, x1,y1, lum;

	sizeh=sizex*.5;
	if (insize<1) insize=1;

	// modify x noise -----------------------------
	if (intype&1)	for (i=0; i<sizex; i++) xnoisev[i]=rndn(grain)-grainh; // noise value for vertical lines

	for    (y = 0; y <sizey; y++) { // for every line
		if (intype&2) ynoise=rndn(grain)-grainh;// rnd tv noise per horizontal line
		   y1=(y-sizeh)*(y-sizeh);
			addy=y*sizey;
		for (x = 0; x <sizex; x++) { // for every pixel in x
			x1=x-sizeh;
			lum=255; // start at full intensity and work mask towards black w/ effects

			if (intype&1) lum+=xnoisev[x]; // x film noise
			if (intype&2) lum+=ynoise;     // y tv noise

			if (intype&4) { // z noise
				lum+=rndn(grain)-grainh;
				//lum=(lum<0)?0:(lum>255)?255:lum;
			}
			if (intype&8) { // vignette
				r=1-pow(sqrt(x1*x1+y1)/insize,2);// 0-1  ** adj sharpness
				if (r<0) r=0;
				lum*=r;
			}
			if ((intype&16)&&(x&1)) lum-=grain;// x grating
			if ((intype&32)&&(y&1)) lum-=grain;// y grating
				
			lum=clampf(lum,0,255); 

			switch(type) {
				case GL_RGBA:
					addr=(x+addy)*4;
					array[addr]=array[addr+1]=array[addr+2]=array[addr+3]=lum;
					break;
				case GL_LUMINANCE_ALPHA:
					addr=(x+addy)*2;
					array[addr]=array[addr+1]=lum;
					break;
			}
		}
	}
	bind();
	glTexSubImage2D(GL_TEXTURE_2D,0, 0,0,sizex,sizey,type,GL_UNSIGNED_BYTE,array);
	lastUpdateTime=Hw::get()->time.time;
}

//-------------------------------------------------------------------------------
//-------------------------------------------------------------------------------
//-------------------------------------------------------------------------------

/*

BITMAPFILEHEADER bmfh;
BITMAPINFOHEADER bmih;
RGBQUAD          aColors[];
BYTE             aBitmapBits[];

typedef struct tagBITMAPINFOHEADER {
    DWORD   biSize;
    LONG    biWidth;
    LONG    biHeight;
    WORD    biPlanes;
    WORD    biBitCount;
    DWORD   biCompression;
    DWORD   biSizeImage;
    LONG    biXPelsPerMeter;
    LONG    biYPelsPerMeter;
    DWORD   biClrUsed;
    DWORD   biClrImportant;
} BITMAPINFOHEADER;

The BITMAPINFOHEADER structure contains information about the dimensions and
color format of a Windows 3.0 or later device-independent bitmap (DIB).

Member          Description

biSize          Specifies the number of bytes required by the
BITMAPINFOHEADER structure.

biWidth         Specifies the width of the bitmap, in pixels. 
biHeight        Specifies the height of the bitmap, in pixels. 

biPlanes        Specifies the number of planes for the target device. This
member must be set to 1.

biBitCount      Specifies the number of bits per pixel. This value must be 1,
4, 8, or 24.

biCompression   Specifies the type of compression for a compressed bitmap. It
can be one of the following values:

Value           Meaning

BI_RGB          Specifies that the bitmap is not compressed. 

BI_RLE8         Specifies a run-length encoded format for bitmaps with 8 bits
per pixel. The compression format is a 2-byte format consisting of a count
byte followed by a byte containing a color index.  For more information, see
the following Comments section.

biSizeImage     Specifies the size, in bytes, of the image. It is valid to
set this member to zero if the bitmap is in the BI_RGB format.

biXPelsPerMeter Specifies the horizontal resolution, in pixels per meter, of
the target device for the bitmap. An application can use this value to select
a bitmap from a resource group that best matches the characteristics of the
current device.

biYPelsPerMeter Specifies the vertical resolution, in pixels per meter, of
the target device for the bitmap.

biClrUsed       Specifies the number of color indexes in the color table
actually used by the bitmap. If this value is zero, the bitmap uses the
maximum number of colors corresponding to the value of the biBitCount member.
For more information on the maximum sizes of the color table, see the
description of the BITMAPINFO structure earlier in this topic.

If the biClrUsed member is nonzero, it specifies the actual number of colors
that the graphics engine or device driver will access if the biBitCount
member is less than 24. If biBitCount is set to 24, biClrUsed specifies the
size of the reference color table used to optimize performance of Windows
color palettes.  If the bitmap is a packed bitmap (that is, a bitmap in which
the bitmap array immediately follows the BITMAPINFO header and which is
referenced by a single pointer), the biClrUsed member must be set to zero or
to the actual size of the color table.

biClrImportant  Specifies the number of color indexes that are considered
important for displaying the bitmap. If this value is zero, all colors are
important.
*/